@@ -1,30 +1,26 @@
 use std::task::{Context, Poll};
-use std::{convert::TryFrom, fmt, future::Future, io, marker, pin::Pin, rc::Rc, time};
+use std::{convert::TryFrom, fmt, future::Future, io, marker, pin::Pin, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
+use ntex::io::{Filter, Io, IoBoxed, RecvError};
 use ntex::service::{Service, ServiceFactory};
 use ntex::time::{sleep, Seconds, Sleep};
-use ntex::util::{join, Pool, PoolId, PoolRef, Ready};
+use ntex::util::{join, ready, Ready};
 
-use crate::error::{MqttError, ProtocolError};
-use crate::io::State;
 use crate::version::{ProtocolVersion, VersionCodec};
-use crate::{v3, v5};
+use crate::{error::MqttError, v3, v5};
 
 /// Mqtt Server
-pub struct MqttServer<Io, V3, V5, Err, InitErr> {
+pub struct MqttServer<V3, V5, Err, InitErr> {
     v3: V3,
     v5: V5,
     handshake_timeout: Seconds,
-    pool: Pool,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
+    _t: marker::PhantomData<(Err, InitErr)>,
 }
 
-impl<Io, Err, InitErr>
+impl<Err, InitErr>
     MqttServer<
-        Io,
-        DefaultProtocolServer<Io, Err, InitErr>,
-        DefaultProtocolServer<Io, Err, InitErr>,
+        DefaultProtocolServer<Err, InitErr>,
+        DefaultProtocolServer<Err, InitErr>,
         Err,
         InitErr,
     >
@@ -34,18 +30,16 @@ impl<Io, Err, InitErr>
         MqttServer {
             v3: DefaultProtocolServer::new(ProtocolVersion::MQTT3),
             v5: DefaultProtocolServer::new(ProtocolVersion::MQTT5),
-            pool: PoolId::P5.pool(),
             handshake_timeout: Seconds::ZERO,
             _t: marker::PhantomData,
         }
     }
 }
 
-impl<Io, Err, InitErr> Default
+impl<Err, InitErr> Default
     for MqttServer<
-        Io,
-        DefaultProtocolServer<Io, Err, InitErr>,
-        DefaultProtocolServer<Io, Err, InitErr>,
+        DefaultProtocolServer<Err, InitErr>,
+        DefaultProtocolServer<Err, InitErr>,
         Err,
         InitErr,
     >
@@ -55,7 +49,7 @@ impl<Io, Err, InitErr> Default
     }
 }
 
-impl<Io, V3, V5, Err, InitErr> MqttServer<Io, V3, V5, Err, InitErr> {
+impl<V3, V5, Err, InitErr> MqttServer<V3, V5, Err, InitErr> {
     /// Set handshake timeout.
     ///
     /// Handshake includes `connect` packet.
@@ -64,30 +58,18 @@ impl<Io, V3, V5, Err, InitErr> MqttServer<Io, V3, V5, Err, InitErr> {
         self.handshake_timeout = timeout;
         self
     }
-
-    /// Set memory pool.
-    ///
-    /// Use specified memory pool for memory allocations. By default P5
-    /// memory pool is used.
-    pub fn memory_pool(mut self, id: PoolId) -> Self {
-        self.pool = id.pool();
-        self
-    }
 }
 
-impl<Io, V3, V5, Err, InitErr> MqttServer<Io, V3, V5, Err, InitErr>
+impl<V3, V5, Err, InitErr> MqttServer<V3, V5, Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     V3: ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
     >,
     V5: ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
@@ -96,12 +78,10 @@ where
     /// Service to handle v3 protocol
     pub fn v3<St, C, Cn, P>(
         self,
-        service: v3::MqttServer<Io, St, C, Cn, P>,
+        service: v3::MqttServer<St, C, Cn, P>,
     ) -> MqttServer<
-        Io,
         impl ServiceFactory<
-            Config = (),
-            Request = (Io, State, Option<Sleep>),
+            (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
             InitError = InitErr,
@@ -113,19 +93,17 @@ where
     where
         St: 'static,
         C: ServiceFactory<
-                Config = (),
-                Request = v3::Handshake<Io>,
-                Response = v3::HandshakeAck<Io, St>,
+                v3::Handshake,
+                Response = v3::HandshakeAck<St>,
                 Error = Err,
                 InitError = InitErr,
             > + 'static,
         Cn: ServiceFactory<
-                Config = v3::Session<St>,
-                Request = v3::ControlMessage<C::Error>,
+                v3::ControlMessage<Err>,
+                v3::Session<St>,
                 Response = v3::ControlResult,
             > + 'static,
-        P: ServiceFactory<Config = v3::Session<St>, Request = v3::Publish, Response = ()>
-            + 'static,
+        P: ServiceFactory<v3::Publish, v3::Session<St>, Response = ()> + 'static,
         C::Error: From<Cn::Error>
             + From<Cn::InitError>
             + From<P::Error>
@@ -133,9 +111,8 @@ where
             + fmt::Debug,
     {
         MqttServer {
-            v3: service.inner_finish(),
+            v3: service.finish(),
             v5: self.v5,
-            pool: self.pool,
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
@@ -144,12 +121,10 @@ where
     /// Service to handle v3 protocol
     pub fn v3_variants(
         self,
-        service: v3::Selector<Io, Err, InitErr>,
+        service: v3::Selector<Err, InitErr>,
     ) -> MqttServer<
-        Io,
         impl ServiceFactory<
-            Config = (),
-            Request = (Io, State, Option<Sleep>),
+            (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
             InitError = InitErr,
@@ -163,9 +138,8 @@ where
         InitErr: 'static,
     {
         MqttServer {
-            v3: service.finish_server(),
+            v3: service,
             v5: self.v5,
-            pool: self.pool,
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
@@ -174,13 +148,11 @@ where
     /// Service to handle v5 protocol
     pub fn v5<St, C, Cn, P>(
         self,
-        service: v5::MqttServer<Io, St, C, Cn, P>,
+        service: v5::MqttServer<St, C, Cn, P>,
     ) -> MqttServer<
-        Io,
         V3,
         impl ServiceFactory<
-            Config = (),
-            Request = (Io, State, Option<Sleep>),
+            (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
             InitError = InitErr,
@@ -191,22 +163,17 @@ where
     where
         St: 'static,
         C: ServiceFactory<
-                Config = (),
-                Request = v5::Handshake<Io>,
-                Response = v5::HandshakeAck<Io, St>,
+                v5::Handshake,
+                Response = v5::HandshakeAck<St>,
                 Error = Err,
                 InitError = InitErr,
             > + 'static,
         Cn: ServiceFactory<
-                Config = v5::Session<St>,
-                Request = v5::ControlMessage<C::Error>,
+                v5::ControlMessage<Err>,
+                v5::Session<St>,
                 Response = v5::ControlResult,
             > + 'static,
-        P: ServiceFactory<
-                Config = v5::Session<St>,
-                Request = v5::Publish,
-                Response = v5::PublishAck,
-            > + 'static,
+        P: ServiceFactory<v5::Publish, v5::Session<St>, Response = v5::PublishAck> + 'static,
         P::Error: fmt::Debug,
         C::Error: From<Cn::Error>
             + From<Cn::InitError>
@@ -217,8 +184,7 @@ where
     {
         MqttServer {
             v3: self.v3,
-            v5: service.inner_finish(),
-            pool: self.pool,
+            v5: service.finish(),
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
@@ -227,13 +193,11 @@ where
     /// Service to handle v5 protocol
     pub fn v5_variants<St, C, Cn, P>(
         self,
-        service: v5::Selector<Io, Err, InitErr>,
+        service: v5::Selector<Err, InitErr>,
     ) -> MqttServer<
-        Io,
         V3,
         impl ServiceFactory<
-            Config = (),
-            Request = (Io, State, Option<Sleep>),
+            (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
             InitError = InitErr,
@@ -247,97 +211,143 @@ where
     {
         MqttServer {
             v3: self.v3,
-            v5: service.finish_server(),
-            pool: self.pool,
+            v5: service,
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
     }
 }
 
-impl<Io, V3, V5, Err, InitErr> ServiceFactory for MqttServer<Io, V3, V5, Err, InitErr>
+impl<V3, V5, Err, InitErr> MqttServer<V3, V5, Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     V3: ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
     >,
     V5: ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
         Response = (),
         Error = MqttError<Err>,
         InitError = InitErr,
     >,
-    V3::Future: 'static,
-    V5::Future: 'static,
 {
-    type Config = ();
-    type Request = Io;
-    type Response = ();
-    type Error = MqttError<Err>;
-    type Service = MqttServerImpl<Io, V3::Service, V5::Service, Err>;
-    type InitError = InitErr;
-    type Future = Pin<
-        Box<
-            dyn Future<
-                Output = Result<MqttServerImpl<Io, V3::Service, V5::Service, Err>, InitErr>,
-            >,
-        >,
-    >;
-
-    fn new_service(&self, _: ()) -> Self::Future {
-        let pool = self.pool.clone();
+    fn create_service(
+        &self,
+    ) -> impl Future<Output = Result<MqttServerImpl<V3::Service, V5::Service, Err>, InitErr>>
+    {
         let handshake_timeout = self.handshake_timeout;
         let fut = join(self.v3.new_service(()), self.v5.new_service(()));
-        Box::pin(async move {
+        async move {
             let (v3, v5) = fut.await;
             let v3 = v3?;
             let v5 = v5?;
             Ok(MqttServerImpl {
                 handlers: Rc::new((v3, v5)),
-                pool,
                 handshake_timeout,
                 _t: marker::PhantomData,
             })
-        })
+        }
+    }
+}
+
+impl<V3, V5, Err, InitErr> ServiceFactory<IoBoxed> for MqttServer<V3, V5, Err, InitErr>
+where
+    V3: ServiceFactory<
+            (IoBoxed, Option<Sleep>),
+            Response = (),
+            Error = MqttError<Err>,
+            InitError = InitErr,
+        > + 'static,
+    V5: ServiceFactory<
+            (IoBoxed, Option<Sleep>),
+            Response = (),
+            Error = MqttError<Err>,
+            InitError = InitErr,
+        > + 'static,
+    Err: 'static,
+    InitErr: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type Service = MqttServerImpl<V3::Service, V5::Service, Err>;
+    type InitError = InitErr;
+    type Future = Pin<
+        Box<
+            dyn Future<Output = Result<MqttServerImpl<V3::Service, V5::Service, Err>, InitErr>>,
+        >,
+    >;
+
+    #[inline]
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+impl<F, V3, V5, Err, InitErr> ServiceFactory<Io<F>> for MqttServer<V3, V5, Err, InitErr>
+where
+    F: Filter,
+    V3: ServiceFactory<
+            (IoBoxed, Option<Sleep>),
+            Response = (),
+            Error = MqttError<Err>,
+            InitError = InitErr,
+        > + 'static,
+    V5: ServiceFactory<
+            (IoBoxed, Option<Sleep>),
+            Response = (),
+            Error = MqttError<Err>,
+            InitError = InitErr,
+        > + 'static,
+    Err: 'static,
+    InitErr: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type Service = MqttServerImpl<V3::Service, V5::Service, Err>;
+    type InitError = InitErr;
+    type Future = Pin<
+        Box<
+            dyn Future<Output = Result<MqttServerImpl<V3::Service, V5::Service, Err>, InitErr>>,
+        >,
+    >;
+
+    #[inline]
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
     }
 }
 
 /// Mqtt Server
-pub struct MqttServerImpl<Io, V3, V5, Err> {
+pub struct MqttServerImpl<V3, V5, Err> {
     handlers: Rc<(V3, V5)>,
     handshake_timeout: Seconds,
-    pool: Pool,
-    _t: marker::PhantomData<(Io, Err)>,
+    _t: marker::PhantomData<Err>,
 }
 
-impl<Io, V3, V5, Err> Service for MqttServerImpl<Io, V3, V5, Err>
+impl<V3, V5, Err> Service<IoBoxed> for MqttServerImpl<V3, V5, Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    V3: Service<Request = (Io, State, Option<Sleep>), Response = (), Error = MqttError<Err>>,
-    V5: Service<Request = (Io, State, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+    V3: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+    V5: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
 {
-    type Request = Io;
     type Response = ();
     type Error = MqttError<Err>;
-    type Future = MqttServerImplResponse<Io, V3, V5, Err>;
+    type Future = MqttServerImplResponse<V3, V5, Err>;
 
+    #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
         let ready1 = self.handlers.0.poll_ready(cx)?.is_ready();
         let ready2 = self.handlers.1.poll_ready(cx)?.is_ready();
-        let ready3 = self.pool.poll_ready(cx).is_ready();
 
-        if ready1 && ready2 && ready3 {
+        if ready1 && ready2 {
             Poll::Ready(Ok(()))
         } else {
             Poll::Pending
         }
     }
 
+    #[inline]
     fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
         let ready1 = self.handlers.0.poll_shutdown(cx, is_error).is_ready();
         let ready2 = self.handlers.1.poll_shutdown(cx, is_error).is_ready();
@@ -349,69 +359,88 @@ where
         }
     }
 
-    fn call(&self, req: Io) -> Self::Future {
-        let pool = self.pool.pool_ref();
+    #[inline]
+    fn call(&self, req: IoBoxed) -> Self::Future {
         let delay = self.handshake_timeout.map(sleep);
 
         MqttServerImplResponse {
             state: MqttServerImplState::Version {
-                item: Some((
-                    req,
-                    State::with_memory_pool(pool),
-                    VersionCodec,
-                    self.handlers.clone(),
-                    delay,
-                )),
+                item: Some((req, VersionCodec, self.handlers.clone(), delay)),
             },
         }
     }
 }
 
+impl<F, V3, V5, Err> Service<Io<F>> for MqttServerImpl<V3, V5, Err>
+where
+    F: Filter,
+    V3: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+    V5: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type Future = MqttServerImplResponse<V3, V5, Err>;
+
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        Service::<IoBoxed>::poll_ready(self, cx)
+    }
+
+    #[inline]
+    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
+        Service::<IoBoxed>::poll_shutdown(self, cx, is_error)
+    }
+
+    #[inline]
+    fn call(&self, io: Io<F>) -> Self::Future {
+        Service::<IoBoxed>::call(self, IoBoxed::from(io))
+    }
+}
+
 pin_project_lite::pin_project! {
-    pub struct MqttServerImplResponse<Io, V3, V5, Err>
+    pub struct MqttServerImplResponse<V3, V5, Err>
     where
         V3: Service<
-            Request = (Io, State, Option<Sleep>),
+            (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
         >,
         V5: Service<
-            Request = (Io, State, Option<Sleep>),
+        (IoBoxed, Option<Sleep>),
             Response = (),
             Error = MqttError<Err>,
         >,
     {
         #[pin]
-        state: MqttServerImplState<Io, V3, V5>,
+        state: MqttServerImplState<V3, V5>,
     }
 }
 
 pin_project_lite::pin_project! {
     #[project = MqttServerImplStateProject]
-    pub(crate) enum MqttServerImplState<Io, V3: Service, V5: Service> {
+    pub(crate) enum MqttServerImplState<V3: Service<(IoBoxed, Option<Sleep>)>, V5: Service<(IoBoxed, Option<Sleep>)>> {
         V3 { #[pin] fut: V3::Future },
         V5 { #[pin] fut: V5::Future },
-        Version { item: Option<(Io, State, VersionCodec, Rc<(V3, V5)>, Option<Sleep>)> },
+        Version { item: Option<(IoBoxed, VersionCodec, Rc<(V3, V5)>, Option<Sleep>)> },
     }
 }
 
-impl<Io, V3, V5, Err> Future for MqttServerImplResponse<Io, V3, V5, Err>
+impl<V3, V5, Err> Future for MqttServerImplResponse<V3, V5, Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    V3: Service<Request = (Io, State, Option<Sleep>), Response = (), Error = MqttError<Err>>,
-    V5: Service<Request = (Io, State, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+    V3: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
+    V5: Service<(IoBoxed, Option<Sleep>), Response = (), Error = MqttError<Err>>,
 {
     type Output = Result<(), MqttError<Err>>;
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        let mut this = self.as_mut().project();
-
         loop {
+            let mut this = self.as_mut().project();
+
             match this.state.project() {
                 MqttServerImplStateProject::V3 { fut } => return fut.poll(cx),
                 MqttServerImplStateProject::V5 { fut } => return fut.poll(cx),
                 MqttServerImplStateProject::Version { ref mut item } => {
-                    if let Some(ref mut delay) = item.as_mut().unwrap().4 {
+                    if let Some(ref mut delay) = item.as_mut().unwrap().3 {
                         match Pin::new(delay).poll(cx) {
                             Poll::Pending => (),
                             Poll::Ready(_) => {
@@ -422,53 +451,62 @@ where
 
                     let st = item.as_mut().unwrap();
 
-                    match st.1.poll_next(&mut st.0, &st.2, cx) {
-                        Poll::Ready(Ok(Some(ver))) => {
-                            let (io, state, _, handlers, delay) = item.take().unwrap();
+                    return match ready!(st.0.poll_recv(&st.1, cx)) {
+                        Ok(ver) => {
+                            let (io, _, handlers, delay) = item.take().unwrap();
                             this = self.as_mut().project();
                             match ver {
                                 ProtocolVersion::MQTT3 => {
                                     this.state.set(MqttServerImplState::V3 {
-                                        fut: handlers.0.call((io, state, delay)),
+                                        fut: handlers.0.call((io, delay)),
                                     })
                                 }
                                 ProtocolVersion::MQTT5 => {
                                     this.state.set(MqttServerImplState::V5 {
-                                        fut: handlers.1.call((io, state, delay)),
+                                        fut: handlers.1.call((io, delay)),
                                     })
                                 }
                             }
                             continue;
                         }
-                        Poll::Ready(Ok(None)) => {
-                            return Poll::Ready(Err(MqttError::Disconnected))
+                        Err(RecvError::KeepAlive | RecvError::Stop) => {
+                            unreachable!()
                         }
-                        Poll::Ready(Err(err)) => return Poll::Ready(Err(MqttError::from(err))),
-                        Poll::Pending => return Poll::Pending,
-                    }
+                        Err(RecvError::WriteBackpressure) => {
+                            ready!(st.0.poll_flush(cx, false))
+                                .map_err(|e| MqttError::Disconnected(Some(e)))?;
+                            continue;
+                        }
+                        Err(RecvError::Decoder(err)) => {
+                            Poll::Ready(Err(MqttError::Protocol(err.into())))
+                        }
+                        Err(RecvError::PeerGone(err)) => {
+                            Poll::Ready(Err(MqttError::Disconnected(err)))
+                        }
+                    };
                 }
             }
         }
     }
 }
 
-pub struct DefaultProtocolServer<Io, Err, InitErr> {
+pub struct DefaultProtocolServer<Err, InitErr> {
     ver: ProtocolVersion,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
+    _t: marker::PhantomData<(Err, InitErr)>,
 }
 
-impl<Io, Err, InitErr> DefaultProtocolServer<Io, Err, InitErr> {
+impl<Err, InitErr> DefaultProtocolServer<Err, InitErr> {
     fn new(ver: ProtocolVersion) -> Self {
         Self { ver, _t: marker::PhantomData }
     }
 }
 
-impl<Io, Err, InitErr> ServiceFactory for DefaultProtocolServer<Io, Err, InitErr> {
-    type Config = ();
-    type Request = (Io, State, Option<Sleep>);
+impl<Err, InitErr> ServiceFactory<(IoBoxed, Option<Sleep>)>
+    for DefaultProtocolServer<Err, InitErr>
+{
     type Response = ();
     type Error = MqttError<Err>;
-    type Service = DefaultProtocolServer<Io, Err, InitErr>;
+    type Service = DefaultProtocolServer<Err, InitErr>;
     type InitError = InitErr;
     type Future = Ready<Self::Service, Self::InitError>;
 
@@ -477,8 +515,7 @@ impl<Io, Err, InitErr> ServiceFactory for DefaultProtocolServer<Io, Err, InitErr
     }
 }
 
-impl<Io, Err, InitErr> Service for DefaultProtocolServer<Io, Err, InitErr> {
-    type Request = (Io, State, Option<Sleep>);
+impl<Err, InitErr> Service<(IoBoxed, Option<Sleep>)> for DefaultProtocolServer<Err, InitErr> {
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Ready<Self::Response, Self::Error>;
@@ -487,8 +524,8 @@ impl<Io, Err, InitErr> Service for DefaultProtocolServer<Io, Err, InitErr> {
         Poll::Ready(Ok(()))
     }
 
-    fn call(&self, _: Self::Request) -> Self::Future {
-        Ready::Err(MqttError::Protocol(ProtocolError::Io(io::Error::new(
+    fn call(&self, _: (IoBoxed, Option<Sleep>)) -> Self::Future {
+        Ready::Err(MqttError::Disconnected(Some(io::Error::new(
             io::ErrorKind::Other,
             format!("Protocol is not supported: {:?}", self.ver),
         ))))
