@@ -2,7 +2,7 @@ use std::task::{Context, Poll};
 use std::{fmt, future::Future, marker::PhantomData, pin::Pin, rc::Rc};
 
 use ntex::codec::{Decoder, Encoder};
-use ntex::io::{DispatchItem, IoBoxed};
+use ntex::io::{DispatchItem, Filter, Io, IoBoxed};
 use ntex::service::{Service, ServiceFactory};
 use ntex::time::{Seconds, Sleep};
 use ntex::util::{select, Either};
@@ -11,25 +11,43 @@ use crate::io::Dispatcher;
 
 type ResponseItem<U> = Option<<U as Encoder>::Item>;
 
-pub(crate) struct FramedService<St, C, T, Codec> {
+pub struct MqttServer<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
     disconnect_timeout: Seconds,
     _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Codec> FramedService<St, C, T, Codec> {
+impl<St, C, T, Codec> MqttServer<St, C, T, Codec> {
     pub(crate) fn new(connect: C, service: T, disconnect_timeout: Seconds) -> Self {
-        FramedService {
-            connect,
-            disconnect_timeout,
-            handler: Rc::new(service),
-            _t: PhantomData,
+        MqttServer { connect, disconnect_timeout, handler: Rc::new(service), _t: PhantomData }
+    }
+}
+
+impl<St, C, T, Codec> MqttServer<St, C, T, Codec>
+where
+    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)>,
+{
+    fn create_service(
+        &self,
+    ) -> impl Future<Output = Result<MqttHandler<St, C::Service, T, Codec>, C::InitError>> {
+        let fut = self.connect.new_service(());
+        let handler = self.handler.clone();
+        let disconnect_timeout = self.disconnect_timeout;
+
+        // create connect service and then create service impl
+        async move {
+            Ok(MqttHandler {
+                handler,
+                disconnect_timeout,
+                connect: fut.await?,
+                _t: PhantomData,
+            })
         }
     }
 }
 
-impl<St, C, T, Codec> ServiceFactory<IoBoxed> for FramedService<St, C, T, Codec>
+impl<St, C, T, Codec> ServiceFactory<IoBoxed> for MqttServer<St, C, T, Codec>
 where
     St: 'static,
     C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
@@ -42,39 +60,77 @@ where
             InitError = C::Error,
         > + 'static,
     Codec: Decoder + Encoder + Clone + 'static,
-    <Codec as Encoder>::Item: 'static,
 {
     type Response = ();
     type Error = C::Error;
     type InitError = C::InitError;
-    type Service = FramedServiceImpl<St, C::Service, T, Codec>;
+    type Service = MqttHandler<St, C::Service, T, Codec>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
-        let fut = self.connect.new_service(());
-        let handler = self.handler.clone();
-        let disconnect_timeout = self.disconnect_timeout;
-
-        // create connect service and then create service impl
-        Box::pin(async move {
-            Ok(FramedServiceImpl {
-                handler,
-                disconnect_timeout,
-                connect: fut.await?,
-                _t: PhantomData,
-            })
-        })
+        Box::pin(self.create_service())
     }
 }
 
-pub(crate) struct FramedServiceImpl<St, C, T, Codec> {
+impl<F, St, C, T, Codec> ServiceFactory<Io<F>> for MqttServer<St, C, T, Codec>
+where
+    F: Filter,
+    St: 'static,
+    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
+    C::Error: fmt::Debug,
+    T: ServiceFactory<
+            DispatchItem<Codec>,
+            St,
+            Response = ResponseItem<Codec>,
+            Error = C::Error,
+            InitError = C::Error,
+        > + 'static,
+    Codec: Decoder + Encoder + Clone + 'static,
+{
+    type Response = ();
+    type Error = C::Error;
+    type InitError = C::InitError;
+    type Service = MqttHandler<St, C::Service, T, Codec>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+impl<St, C, T, Codec> ServiceFactory<(IoBoxed, Option<Sleep>)> for MqttServer<St, C, T, Codec>
+where
+    St: 'static,
+    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
+    C::Error: fmt::Debug,
+    T: ServiceFactory<
+            DispatchItem<Codec>,
+            St,
+            Response = ResponseItem<Codec>,
+            Error = C::Error,
+            InitError = C::Error,
+        > + 'static,
+    Codec: Decoder + Encoder + Clone + 'static,
+{
+    type Response = ();
+    type Error = C::Error;
+    type InitError = C::InitError;
+    type Service = MqttHandler<St, C::Service, T, Codec>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+pub struct MqttHandler<St, C, T, Codec> {
     connect: C,
     handler: Rc<T>,
     disconnect_timeout: Seconds,
     _t: PhantomData<(St, Codec)>,
 }
 
-impl<St, C, T, Codec> Service<IoBoxed> for FramedServiceImpl<St, C, T, Codec>
+impl<St, C, T, Codec> Service<IoBoxed> for MqttHandler<St, C, T, Codec>
 where
     St: 'static,
     C: Service<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
@@ -104,8 +160,6 @@ where
 
     #[inline]
     fn call(&self, req: IoBoxed) -> Self::Future {
-        log::trace!("Start connection handshake");
-
         let handler = self.handler.clone();
         let timeout = self.disconnect_timeout;
         let handshake = self.connect.call(req);
@@ -128,29 +182,11 @@ where
     }
 }
 
-pub(crate) struct FramedService2<St, C, T, Codec> {
-    connect: C,
-    handler: Rc<T>,
-    disconnect_timeout: Seconds,
-    _t: PhantomData<(St, Codec)>,
-}
-
-impl<St, C, T, Codec> FramedService2<St, C, T, Codec> {
-    pub(crate) fn new(connect: C, service: T, disconnect_timeout: Seconds) -> Self {
-        FramedService2 {
-            connect,
-            disconnect_timeout,
-            handler: Rc::new(service),
-            _t: PhantomData,
-        }
-    }
-}
-
-impl<St, C, T, Codec> ServiceFactory<(IoBoxed, Option<Sleep>)>
-    for FramedService2<St, C, T, Codec>
+impl<F, St, C, T, Codec> Service<Io<F>> for MqttHandler<St, C, T, Codec>
 where
+    F: Filter,
     St: 'static,
-    C: ServiceFactory<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
+    C: Service<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
     C::Error: fmt::Debug,
     T: ServiceFactory<
             DispatchItem<Codec>,
@@ -163,35 +199,25 @@ where
 {
     type Response = ();
     type Error = C::Error;
-    type InitError = C::InitError;
-    type Service = FramedServiceImpl2<St, C::Service, T, Codec>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+    type Future = Pin<Box<dyn Future<Output = Result<(), Self::Error>>>>;
 
-    fn new_service(&self, _: ()) -> Self::Future {
-        let fut = self.connect.new_service(());
-        let handler = self.handler.clone();
-        let disconnect_timeout = self.disconnect_timeout;
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        self.connect.poll_ready(cx)
+    }
 
-        // create connect service and then create service impl
-        Box::pin(async move {
-            Ok(FramedServiceImpl2 {
-                handler,
-                disconnect_timeout,
-                connect: fut.await?,
-                _t: PhantomData,
-            })
-        })
+    #[inline]
+    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
+        self.connect.poll_shutdown(cx, is_error)
+    }
+
+    #[inline]
+    fn call(&self, io: Io<F>) -> Self::Future {
+        Service::<IoBoxed>::call(self, IoBoxed::from(io))
     }
 }
 
-pub(crate) struct FramedServiceImpl2<St, C, T, Codec> {
-    connect: C,
-    handler: Rc<T>,
-    disconnect_timeout: Seconds,
-    _t: PhantomData<(St, Codec)>,
-}
-
-impl<St, C, T, Codec> Service<(IoBoxed, Option<Sleep>)> for FramedServiceImpl2<St, C, T, Codec>
+impl<St, C, T, Codec> Service<(IoBoxed, Option<Sleep>)> for MqttHandler<St, C, T, Codec>
 where
     St: 'static,
     C: Service<IoBoxed, Response = (IoBoxed, Codec, St, Seconds)> + 'static,
@@ -220,12 +246,10 @@ where
     }
 
     #[inline]
-    fn call(&self, (req, delay): (IoBoxed, Option<Sleep>)) -> Self::Future {
-        log::trace!("Start connection handshake");
-
+    fn call(&self, (io, delay): (IoBoxed, Option<Sleep>)) -> Self::Future {
         let handler = self.handler.clone();
         let timeout = self.disconnect_timeout;
-        let handshake = self.connect.call(req);
+        let handshake = self.connect.call(io);
 
         Box::pin(async move {
             let (io, codec, ka, handler) = if let Some(delay) = delay {
