@@ -1,45 +1,37 @@
-use std::{fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll, time};
+use std::{fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::service::{apply_fn_factory, boxed, IntoServiceFactory, Service, ServiceFactory};
+use ntex::io::{Filter, Io, IoBoxed};
+use ntex::service::{boxed, Service, ServiceFactory};
 use ntex::time::{sleep, Seconds, Sleep};
-use ntex::util::{timeout::Timeout, timeout::TimeoutError, Either, PoolId, Ready};
+use ntex::util::Either;
 
 use crate::error::{MqttError, ProtocolError};
-use crate::io::{DispatchItem, State};
 
 use super::control::{ControlMessage, ControlResult};
-use super::default::{DefaultControlService, DefaultPublishService};
 use super::handshake::{Handshake, HandshakeAck};
 use super::shared::{MqttShared, MqttSinkPool};
-use super::{codec as mqtt, dispatcher::factory, MqttServer, MqttSink, Publish, Session};
+use super::{codec as mqtt, MqttServer, Publish, Session};
 
-pub(crate) type SelectItem<Io> = (Handshake<Io>, State, Option<Sleep>);
+pub(crate) type SelectItem = (Handshake, Option<Sleep>);
 
-type ServerFactory<Io, Err, InitErr> = boxed::BoxServiceFactory<
-    (),
-    SelectItem<Io>,
-    Either<SelectItem<Io>, ()>,
-    MqttError<Err>,
-    InitErr,
->;
+type ServerFactory<Err, InitErr> =
+    boxed::BoxServiceFactory<(), SelectItem, Either<SelectItem, ()>, MqttError<Err>, InitErr>;
 
-type Server<Io, Err> =
-    boxed::BoxService<SelectItem<Io>, Either<SelectItem<Io>, ()>, MqttError<Err>>;
+type Server<Err> = boxed::BoxService<SelectItem, Either<SelectItem, ()>, MqttError<Err>>;
 
 /// Mqtt server selector
 ///
 /// Selector allows to choose different mqtt server impls depends on
 /// connectt packet.
-pub struct Selector<Io, Err, InitErr> {
-    servers: Vec<ServerFactory<Io, Err, InitErr>>,
+pub struct Selector<Err, InitErr> {
+    servers: Vec<ServerFactory<Err, InitErr>>,
     max_size: u32,
     handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
+    _t: marker::PhantomData<(Err, InitErr)>,
 }
 
-impl<Io, Err, InitErr> Selector<Io, Err, InitErr> {
+impl<Err, InitErr> Selector<Err, InitErr> {
     #[allow(clippy::new_without_default)]
     pub fn new() -> Self {
         Selector {
@@ -52,9 +44,8 @@ impl<Io, Err, InitErr> Selector<Io, Err, InitErr> {
     }
 }
 
-impl<Io, Err, InitErr> Selector<Io, Err, InitErr>
+impl<Err, InitErr> Selector<Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
     InitErr: 'static,
 {
@@ -76,38 +67,25 @@ where
         self
     }
 
-    /// Set memory pool.
-    ///
-    /// Use specified memory pool for memory allocations. By default P5
-    /// memory pool is used.
-    pub fn memory_pool(self, id: PoolId) -> Self {
-        self.pool.pool.set(id.pool_ref());
-        self
-    }
-
     /// Add server variant
     pub fn variant<F, R, St, C, Cn, P>(
         mut self,
         check: F,
-        mut server: MqttServer<Io, St, C, Cn, P>,
+        mut server: MqttServer<St, C, Cn, P>,
     ) -> Self
     where
-        F: Fn(&Handshake<Io>) -> R + 'static,
+        F: Fn(&Handshake) -> R + 'static,
         R: Future<Output = Result<bool, Err>> + 'static,
         St: 'static,
         C: ServiceFactory<
-                Config = (),
-                Request = Handshake<Io>,
-                Response = HandshakeAck<Io, St>,
+                Handshake,
+                Response = HandshakeAck<St>,
                 Error = Err,
                 InitError = InitErr,
             > + 'static,
-        Cn: ServiceFactory<
-                Config = Session<St>,
-                Request = ControlMessage<C::Error>,
-                Response = ControlResult,
-            > + 'static,
-        P: ServiceFactory<Config = Session<St>, Request = Publish, Response = ()> + 'static,
+        Cn: ServiceFactory<ControlMessage<Err>, Session<St>, Response = ControlResult>
+            + 'static,
+        P: ServiceFactory<Publish, Session<St>, Response = ()> + 'static,
         C::Error: From<Cn::Error>
             + From<Cn::InitError>
             + From<P::Error>
@@ -118,69 +96,114 @@ where
         self.servers.push(boxed::factory(server.finish_selector(check)));
         self
     }
-
-    /// Set service to handle publish packets and create mqtt server factory
-    pub(crate) fn finish_server(
-        self,
-    ) -> impl ServiceFactory<
-        Config = (),
-        Request = (Io, State, Option<Sleep>),
-        Response = (),
-        Error = MqttError<Err>,
-        InitError = InitErr,
-    > {
-        Selector2 {
-            servers: self.servers,
-            max_size: self.max_size,
-            pool: self.pool,
-            _t: marker::PhantomData,
-        }
-    }
 }
 
-impl<Io, Err, InitErr> ServiceFactory for Selector<Io, Err, InitErr>
+impl<Err, InitErr> Selector<Err, InitErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
     InitErr: 'static,
 {
-    type Config = ();
-    type Request = Io;
-    type Response = ();
-    type Error = MqttError<Err>;
-    type InitError = InitErr;
-    type Service = SelectorService<Io, Err>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
-
-    fn new_service(&self, _: ()) -> Self::Future {
+    fn create_service(&self) -> impl Future<Output = Result<SelectorService<Err>, InitErr>> {
         let futs: Vec<_> = self.servers.iter().map(|srv| srv.new_service(())).collect();
         let max_size = self.max_size;
         let handshake_timeout = self.handshake_timeout;
         let pool = self.pool.clone();
 
-        Box::pin(async move {
+        async move {
             let mut servers = Vec::new();
             for fut in futs {
                 servers.push(fut.await?);
             }
             Ok(SelectorService { max_size, handshake_timeout, pool, servers: Rc::new(servers) })
-        })
+        }
     }
 }
 
-pub struct SelectorService<Io, Err> {
-    servers: Rc<Vec<Server<Io, Err>>>,
+impl<F, Err, InitErr> ServiceFactory<Io<F>> for Selector<Err, InitErr>
+where
+    F: Filter,
+    Err: 'static,
+    InitErr: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type InitError = InitErr;
+    type Service = SelectorService<Err>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+impl<Err, InitErr> ServiceFactory<IoBoxed> for Selector<Err, InitErr>
+where
+    Err: 'static,
+    InitErr: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type InitError = InitErr;
+    type Service = SelectorService<Err>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+impl<Err, InitErr> ServiceFactory<(IoBoxed, Option<Sleep>)> for Selector<Err, InitErr>
+where
+    Err: 'static,
+    InitErr: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type InitError = InitErr;
+    type Service = SelectorService<Err>;
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        Box::pin(self.create_service())
+    }
+}
+
+pub struct SelectorService<Err> {
+    servers: Rc<Vec<Server<Err>>>,
     max_size: u32,
     handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 }
 
-impl<Io, Err> Service for SelectorService<Io, Err>
+impl<F, Err> Service<Io<F>> for SelectorService<Err>
+where
+    F: Filter,
+    Err: 'static,
+{
+    type Response = ();
+    type Error = MqttError<Err>;
+    type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
+
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        Service::<IoBoxed>::poll_ready(self, cx)
+    }
+
+    #[inline]
+    fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
+        Service::<IoBoxed>::poll_shutdown(self, cx, is_error)
+    }
+
+    #[inline]
+    fn call(&self, io: Io<F>) -> Self::Future {
+        Service::<IoBoxed>::call(self, IoBoxed::from(io))
+    }
+}
+
+impl<Err> Service<IoBoxed> for SelectorService<Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
 {
-    type Request = Io;
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
@@ -212,11 +235,10 @@ where
     }
 
     #[inline]
-    fn call(&self, mut io: Io) -> Self::Future {
+    fn call(&self, io: IoBoxed) -> Self::Future {
         let servers = self.servers.clone();
-        let state = State::with_memory_pool(self.pool.pool.get());
         let shared = Rc::new(MqttShared::new(
-            state.clone(),
+            io.clone(),
             mqtt::Codec::default().max_size(self.max_size),
             16,
             self.pool.clone(),
@@ -225,18 +247,16 @@ where
 
         Box::pin(async move {
             // read first packet
-            let packet = state
-                .next(&mut io, &shared.codec)
+            let packet = io
+                .recv(&shared.codec)
                 .await
                 .map_err(|err| {
                     log::trace!("Error is received during mqtt handshake: {:?}", err);
                     MqttError::from(err)
-                })
-                .and_then(|res| {
-                    res.ok_or_else(|| {
-                        log::trace!("Server mqtt is disconnected during handshake");
-                        MqttError::Disconnected
-                    })
+                })?
+                .ok_or_else(|| {
+                    log::trace!("Server mqtt is disconnected during handshake");
+                    MqttError::Disconnected(None)
                 })?;
 
             let connect = match packet {
@@ -251,7 +271,7 @@ where
             };
 
             // call servers
-            let mut item = (Handshake::new(connect, io, shared), state, delay);
+            let mut item = (Handshake::new(connect, io, shared), delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
@@ -266,89 +286,29 @@ where
     }
 }
 
-pub(crate) struct Selector2<Io, Err, InitErr> {
-    servers: Vec<ServerFactory<Io, Err, InitErr>>,
-    max_size: u32,
-    pool: Rc<MqttSinkPool>,
-    _t: marker::PhantomData<(Io, Err, InitErr)>,
-}
-
-impl<Io, Err, InitErr> ServiceFactory for Selector2<Io, Err, InitErr>
+impl<Err> Service<(IoBoxed, Option<Sleep>)> for SelectorService<Err>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    Err: 'static,
-    InitErr: 'static,
-{
-    type Config = ();
-    type Request = (Io, State, Option<Sleep>);
-    type Response = ();
-    type Error = MqttError<Err>;
-    type InitError = InitErr;
-    type Service = SelectorService2<Io, Err>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
-
-    fn new_service(&self, _: ()) -> Self::Future {
-        let futs: Vec<_> = self.servers.iter().map(|srv| srv.new_service(())).collect();
-        let max_size = self.max_size;
-        let pool = self.pool.clone();
-
-        Box::pin(async move {
-            let mut servers = Vec::new();
-            for fut in futs {
-                servers.push(fut.await?);
-            }
-            Ok(SelectorService2 { max_size, pool, servers: Rc::new(servers) })
-        })
-    }
-}
-
-pub(crate) struct SelectorService2<Io, Err> {
-    servers: Rc<Vec<Server<Io, Err>>>,
-    max_size: u32,
-    pool: Rc<MqttSinkPool>,
-}
-
-impl<Io, Err> Service for SelectorService2<Io, Err>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: 'static,
 {
-    type Request = (Io, State, Option<Sleep>);
     type Response = ();
     type Error = MqttError<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<(), MqttError<Err>>>>>;
 
     #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let mut ready = true;
-        for srv in self.servers.iter() {
-            ready &= srv.poll_ready(cx)?.is_ready();
-        }
-        if ready {
-            Poll::Ready(Ok(()))
-        } else {
-            Poll::Pending
-        }
+        Service::<IoBoxed>::poll_ready(self, cx)
     }
 
     #[inline]
     fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
-        let mut ready = true;
-        for srv in self.servers.iter() {
-            ready &= srv.poll_shutdown(cx, is_error).is_ready()
-        }
-        if ready {
-            Poll::Ready(())
-        } else {
-            Poll::Pending
-        }
+        Service::<IoBoxed>::poll_shutdown(self, cx, is_error)
     }
 
     #[inline]
-    fn call(&self, (mut io, state, delay): Self::Request) -> Self::Future {
+    fn call(&self, (io, delay): (IoBoxed, Option<Sleep>)) -> Self::Future {
         let servers = self.servers.clone();
         let shared = Rc::new(MqttShared::new(
-            state.clone(),
+            io.get_ref(),
             mqtt::Codec::default().max_size(self.max_size),
             16,
             self.pool.clone(),
@@ -356,18 +316,16 @@ where
 
         Box::pin(async move {
             // read first packet
-            let packet = state
-                .next(&mut io, &shared.codec)
+            let packet = io
+                .recv(&shared.codec)
                 .await
                 .map_err(|err| {
                     log::trace!("Error is received during mqtt handshake: {:?}", err);
                     MqttError::from(err)
-                })
-                .and_then(|res| {
-                    res.ok_or_else(|| {
-                        log::trace!("Server mqtt is disconnected during handshake");
-                        MqttError::Disconnected
-                    })
+                })?
+                .ok_or_else(|| {
+                    log::trace!("Server mqtt is disconnected during handshake");
+                    MqttError::Disconnected(None)
                 })?;
 
             let connect = match packet {
@@ -382,7 +340,7 @@ where
             };
 
             // call servers
-            let mut item = (Handshake::new(connect, io, shared), state, delay);
+            let mut item = (Handshake::new(connect, io, shared), delay);
             for srv in servers.iter() {
                 match srv.call(item).await? {
                     Either::Left(result) => {
