@@ -2,13 +2,14 @@ use std::cell::RefCell;
 use std::task::{Context, Poll};
 use std::{future::Future, marker::PhantomData, num::NonZeroU16, pin::Pin, rc::Rc};
 
+use ntex::io::DispatchItem;
 use ntex::service::Service;
 use ntex::util::{buffer::BufferService, inflight::InFlightService, Either, HashSet, Ready};
 
 use crate::error::{MqttError, ProtocolError};
+use crate::types::packet_type;
 use crate::v5::shared::{Ack, MqttShared};
 use crate::v5::{codec, publish::Publish, publish::PublishAck, sink::MqttSink};
-use crate::{io::DispatchItem, types::packet_type};
 
 use super::control::{ControlMessage, ControlResult};
 
@@ -19,19 +20,15 @@ pub(super) fn create_dispatcher<T, C, E>(
     max_topic_alias: u16,
     publish: T,
     control: C,
-) -> impl Service<
-    Request = DispatchItem<Rc<MqttShared>>,
-    Response = Option<codec::Packet>,
-    Error = MqttError<E>,
->
+) -> impl Service<DispatchItem<Rc<MqttShared>>, Response = Option<codec::Packet>, Error = MqttError<E>>
 where
     E: From<T::Error> + 'static,
-    T: Service<Request = Publish, Response = Either<Publish, PublishAck>, Error = E> + 'static,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
+    T: Service<Publish, Response = Either<Publish, PublishAck>, Error = E> + 'static,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
 {
     let control = BufferService::new(
         16,
-        || MqttError::<C::Error>::Disconnected,
+        || MqttError::<C::Error>::Disconnected(None),
         // limit number of in-flight messages
         InFlightService::new(1, control.map_err(MqttError::Service)),
     );
@@ -40,7 +37,7 @@ where
 }
 
 /// Mqtt protocol dispatcher
-pub(crate) struct Dispatcher<T, C: Service, E> {
+pub(crate) struct Dispatcher<T, C: Service<ControlMessage<E>>, E> {
     publish: T,
     shutdown: RefCell<Option<Pin<Box<C::Future>>>>,
     max_receive: usize,
@@ -62,8 +59,8 @@ struct PublishInfo {
 
 impl<T, C, E> Dispatcher<T, C, E>
 where
-    T: Service<Request = Publish, Response = Either<Publish, PublishAck>, Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    T: Service<Publish, Response = Either<Publish, PublishAck>, Error = E>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     fn new(
         sink: MqttSink,
@@ -90,13 +87,12 @@ where
     }
 }
 
-impl<T, C, E> Service for Dispatcher<T, C, E>
+impl<T, C, E> Service<DispatchItem<Rc<MqttShared>>> for Dispatcher<T, C, E>
 where
-    T: Service<Request = Publish, Response = Either<Publish, PublishAck>, Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    T: Service<Publish, Response = Either<Publish, PublishAck>, Error = E>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
     C::Future: 'static,
 {
-    type Request = DispatchItem<Rc<MqttShared>>;
     type Response = Option<codec::Packet>;
     type Error = MqttError<E>;
     type Future = Either<
@@ -133,7 +129,7 @@ where
         }
     }
 
-    fn call(&self, request: Self::Request) -> Self::Future {
+    fn call(&self, request: DispatchItem<Rc<MqttShared>>) -> Self::Future {
         log::trace!("Dispatch packet: {:#?}", request);
 
         match request {
@@ -290,10 +286,9 @@ where
                     &self.inner,
                 )))
             }
-            DispatchItem::IoError(err) => Either::Right(Either::Right(ControlResponse::new(
-                ControlMessage::proto_error(ProtocolError::Io(err)),
-                &self.inner,
-            ))),
+            DispatchItem::Disconnect(err) => Either::Right(Either::Right(
+                ControlResponse::new(ControlMessage::peer_gone(err), &self.inner),
+            )),
             DispatchItem::KeepAliveTimeout => {
                 Either::Right(Either::Right(ControlResponse::new(
                     ControlMessage::proto_error(ProtocolError::KeepAliveTimeout),
@@ -309,7 +304,7 @@ where
 
 pin_project_lite::pin_project! {
     /// Publish service response future
-    pub(crate) struct PublishResponse<T: Service, C: Service, E> {
+    pub(crate) struct PublishResponse<T: Service<Publish>, C: Service<ControlMessage<E>>, E> {
         #[pin]
         state: PublishResponseState<T, C, E>,
         packet_id: u16,
@@ -320,7 +315,7 @@ pin_project_lite::pin_project! {
 
 pin_project_lite::pin_project! {
     #[project = PublishResponseStateProject]
-    enum PublishResponseState<T: Service, C: Service, E> {
+    enum PublishResponseState<T: Service<Publish>, C: Service<ControlMessage<E>>, E> {
         Publish { #[pin] fut: T::Future },
         Control { #[pin] fut: ControlResponse<C, E> },
     }
@@ -328,8 +323,8 @@ pin_project_lite::pin_project! {
 
 impl<T, C, E> Future for PublishResponse<T, C, E>
 where
-    T: Service<Request = Publish, Response = Either<Publish, PublishAck>, Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    T: Service<Publish, Response = Either<Publish, PublishAck>, Error = E>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     type Output = Result<Option<codec::Packet>, MqttError<E>>;
 
@@ -381,7 +376,7 @@ where
 
 pin_project_lite::pin_project! {
     /// Control service response future
-    pub(crate) struct ControlResponse<C: Service, E>
+    pub(crate) struct ControlResponse<C: Service<ControlMessage<E>>, E>
     {
         #[pin]
         fut: C::Future,
@@ -392,9 +387,9 @@ pin_project_lite::pin_project! {
     }
 }
 
-impl<C: Service, E> ControlResponse<C, E>
+impl<C, E> ControlResponse<C, E>
 where
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     #[allow(clippy::match_like_matches_macro)]
     fn new(pkt: ControlMessage<E>, inner: &Rc<Inner<C>>) -> Self {
@@ -420,7 +415,7 @@ where
 
 impl<C, E> Future for ControlResponse<C, E>
 where
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     type Output = Result<Option<codec::Packet>, MqttError<E>>;
 
